package be.iminds.ilabt.jfed.restapi.dao;

import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Log;
import be.iminds.ilabt.jfed.restapi.debugdata.ExperimentDD;
import be.iminds.ilabt.jfed.restapi.debugdata.ExperimentDDBuilder;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.dropwizard.jackson.Jackson;
import org.skife.jdbi.v2.sqlobject.Bind;
import org.skife.jdbi.v2.sqlobject.BindBean;
import org.skife.jdbi.v2.sqlobject.SqlQuery;
import org.skife.jdbi.v2.sqlobject.SqlUpdate;
import org.skife.jdbi.v2.sqlobject.customizers.RegisterMapper;

import javax.validation.constraints.NotNull;
import java.sql.Timestamp;

@RegisterMapper(ExperimentDDMapper.class)
public abstract class DebugDataDao {
    @SqlQuery("SELECT experiment_id,json " +
            " FROM experiment_debug_data" +
            " WHERE experiment_id=:id")
    public abstract ExperimentDDBuilder getExperimentDD(@NotNull @Bind("id") Long id);

    @SqlUpdate("UPDATE experiment_debug_data " +
            "SET json=:json " +
            " WHERE experiment_id=:id")
    public abstract int update(@NotNull @Bind("id") Long id,
                               @NotNull @Bind("json") String json);

    @SqlUpdate("INSERT INTO experiment_debug_data (experiment_id,json)" +
            "VALUES (:id,:json)")
    public abstract int insert(@NotNull @Bind("id") Long id,
                               @NotNull @Bind("json") String json);



    private static final ObjectMapper MAPPER;
    static {
        MAPPER = Jackson.newObjectMapper();
        //do notbe strict on unknown properties
        MAPPER.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
    }

    public int insert(@NotNull ExperimentDDBuilder experimentDDBuilder) {
        return insert(experimentDDBuilder.create());
    }
    public int insert(@NotNull ExperimentDD experimentDD) {
        assert experimentDD != null;
        String json;
        try {
            json = MAPPER.writeValueAsString(experimentDD);
        } catch (JsonProcessingException e) {
            throw new RuntimeException("Failed to convert ExperimentDD to JSON", e);
        }
        return insert(experimentDD.getId(), json);
    }
    
    public int update(@NotNull ExperimentDDBuilder experimentDDBuilder) {
        return update(experimentDDBuilder.create());
    }
    public int update(@NotNull ExperimentDD experimentDD) {
        assert experimentDD != null;
        String json;
        try {
            json = MAPPER.writeValueAsString(experimentDD);
        } catch (JsonProcessingException e) {
            throw new RuntimeException("Failed to convert ExperimentDD to JSON", e);
        }
        return update(experimentDD.getId(), json);
    }


//    //dummy implementation for first test, which will rely only on cache
//
//    public ExperimentDDBuilder getExperimentDD(@NotNull @Bind("id") Long id) {
//        return null;
//    }
//
//    public int update(ExperimentDD experimentDD) {
//        return 0;
//    }
//
//    public int insert(ExperimentDD experimentDD) {
//        return 0;
//    }
}
